<?php

namespace Drupal\relatedbyterms\Controller;

use Drupal\Core\Controller\ControllerBase;

/**
 * Main module controller.
 */
class RelatedByTermsController extends ControllerBase {

  protected $currentConfig;

  /**
   * Class cronstuctor.
   */
  public function __construct() {
    $this->currentConfig = \Drupal::configFactory()->getEditable('relatedbyterms.settings');;
  }

  /**
   * Get a list of related nodes.
   *
   * You can call this function directly from your module
   * to get a list of related nodes.
   *
   * @param string $nid
   *   Node id to look for related content.
   * @param string $langcode
   *   Langcode to use to recover content. Defaults to user selected.
   * @param int $limit
   *   Max number of nodes to display. Defaults to the value configured
   *   in the module settings page.
   *
   * @return array
   *   Array with a list of nid of related contents.
   */
  public function getRelatedNodes($nid, $langcode = NULL, $limit = -1) {
    $nodes = array();

    if (empty($langcode)) {
      $langcode = $this->languageManager()->getCurrentLanguage()->getId();
    }

    if ($limit == -1) {
      $limit = $this->getElementsDisplayed();
    }

    $query = $this->getQuery($nid, $langcode, $limit);
    $result = $query->execute();

    while ($record = $result->fetchAssoc()) {
      $nodes[] = $record['nid'];
    }

    return $nodes;
  }

  /**
   * Create DB Query.
   */
  protected function getQuery($nid, $langcode, $limit) {

    $subquery = db_select('taxonomy_index', 't1');
    $subquery->condition('nid', $nid);
    $subquery->addField('t1', 'tid');

    $query = db_select('taxonomy_index', 't');
    $query->join('node', 'n', 'n.nid = t.nid');
    $query->addTag('relatedbyterms_count');
    $query->addTag('node_access');
    $query->condition('n.langcode', $langcode);
    $query->condition('t.tid', $subquery, 'IN');
    $query->condition('t.nid', $nid, '<>');
    $query->addField('t', 'nid');
    $query->addExpression('count(\'t.nid\')', 'count');
    $query->orderBy('count', 'DESC');
    $query->orderBy('nid', 'DESC');
    $query->groupBy('t.nid');

    if ($limit > 0) {
      $query->range(0, $limit);
    }

    return $query;
  }

  /**
   * Returns the number of elements to show.
   *
   * From the configuration values. Can be set in the module config page.
   */
  public function getElementsDisplayed() {
    return $this->currentConfig->get('relatedbyterms.elements_displayed');
  }

  /**
   * Sets the number of elements to show.
   */
  public function setElementsDisplayed($limit) {
    $this->currentConfig->set('relatedbyterms.elements_displayed', $limit);
    $this->currentConfig->save();
  }

  /**
   * Returns the display mode of the elements.
   *
   * From the configuration values. Can be set in the module config page.
   */
  public function getDisplayMode() {
    return $this->currentConfig->get('relatedbyterms.display_mode');
  }

  /**
   * Sets the display mode of the elements.
   */
  public function setDisplayMode($displayMode) {
    $this->currentConfig->set('relatedbyterms.display_mode', $displayMode);
    $this->currentConfig->save();
  }

  /**
   * Returns the default block title.
   */
  public function getDefaultTitle() {
    return $this->currentConfig->get('relatedbyterms.block_title');
  }

  /**
   * Sets the default block title.
   */
  public function setDefaultTitle($defaultTitle) {
    $this->currentConfig->set('relatedbyterms.block_title', $defaultTitle);
    $this->currentConfig->save();
  }

}
