<?php

namespace Drupal\relatedbyterms\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\relatedbyterms\Controller\RelatedByTermsController;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Related by terms configuration form class.
 */
class RelatedByTermsForm extends ConfigFormBase {

  protected $controller;

  /**
   * Class cronstuctor.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    parent::__construct($config_factory);
    $this->controller = new RelatedByTermsController();
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'relatedbyterms_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $form['elements_displayed'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Number of nodes'),
      '#default_value' => $this->controller->getElementsDisplayed(),
      '#description' => $this->t('Number of nodes to show in the block. Use 0 for no limit.'),
    );

    $form['display_mode'] = array(
      '#type' => 'select',
      '#title' => t('Display type'),
      '#options' => $this->getViewModes(),
      '#required' => TRUE,
      '#default_value' => $this->controller->getDisplayMode(),
      '#description' => t('This is the view mode that will be used to render the elements in the block.'),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save data.
    $this->controller->setElementsDisplayed($form_state->getValue('elements_displayed'));
    $this->controller->setDisplayMode($form_state->getValue('display_mode'));

    // After a settings change, invalidate the cache for this block.
    \Drupal::service('cache_tags.invalidator')->invalidateTags(array('config:block.block.relatedbytermsblock'));

    return parent::submitForm($form, $form_state);
  }

  /**
   * Get a list of view modes.
   */
  protected function getViewModes() {
    $node_view_modes = \Drupal::service('entity_display.repository')->getViewModes('node');
    $view_modes = array();

    foreach ($node_view_modes as $key => $node_view) {
      $view_modes[$key] = $node_view['label'];
    }

    return $view_modes;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'relatedbyterms.settings',
    ];
  }

}
