<?php

namespace Drupal\relatedbyterms\Plugin\Block;

use Drupal;
use Drupal\Core\Block\BlockBase;
use Drupal\Component\Plugin\ContextAwarePluginInterface;
use Drupal\node\Entity\Node;
use Drupal\relatedbyterms\Controller\RelatedByTermsController;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Provides a 'Hello' Block.
 *
 * @Block(
 *   id = "relatedbyterms_block",
 *   admin_label = @Translation("Related by Terms block"),
 *   category = @Translation("Related by Terms"),
 *   context = {
 *     "node" = @ContextDefinition("entity:node", label = @Translation("Node"))
 *   }
 * )
 */
class RelatedByTermsBlock extends BlockBase implements ContextAwarePluginInterface {

  /**
   * {@inheritdoc}
   */
  public function build() {
    return $this->getContents();
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $controller = new RelatedByTermsController();
    $base_config = $this->baseConfigurationDefaults();
    $base_config['label'] = $controller->getDefaultTitle();

    return $base_config;
  }

  /**
   * Returns block contents.
   */
  public function getContents() {

    $controller = new RelatedByTermsController();
    $display_mode = $controller->getDisplayMode();
    $current_node = $this->getContextValue('node');
    $nids = $controller->getRelatedNodes($current_node->id());

    $nodes = Node::loadMultiple($nids);
    $build = Drupal::entityTypeManager()->getViewBuilder('node')->viewMultiple($nodes, $display_mode);

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    if ($account->hasPermission('access content')) {
      return AccessResult::allowed();
    }
    else {
      return AccessResult::forbidden();
    }
  }

}
